#include "preferences.h"

// General
cfg_int bpm_config_bpm_precision(guid_bpm_config_bpm_precision, BPM_PRECISION_1);
cfg_string bpm_config_bpm_tag(guid_bpm_config_bpm_tag, "BPM");
// Auto
cfg_int bpm_config_seconds_to_read(guid_bpm_config_seconds_to_read, 10);
cfg_int bpm_config_num_bpms_to_calc(guid_bpm_config_num_bpms_to_calc, 10);
cfg_int bpm_config_offset_pct_min(guid_bpm_config_offset_pct_min, 20);
cfg_int bpm_config_offset_pct_max(guid_bpm_config_offset_pct_max, 80);
cfg_int bpm_config_fft_window_size(guid_bpm_config_fft_window_size, FFT_SIZE_2048);
cfg_int bpm_config_fft_window_slide(guid_bpm_config_fft_window_slide, FFT_SLIDE_AUTO);
cfg_int bpm_config_fft_window_type(guid_bpm_config_window_type, FFT_WINDOW_HAMMING);
cfg_int bpm_config_bpm_min(guid_bpm_config_bpm_min, 75);
cfg_int bpm_config_bpm_max(guid_bpm_config_bpm_max, 195);
cfg_bool bpm_config_interpolate_flux(guid_bpm_config_interpolate_flux, true);
// Manual
cfg_int bpm_config_taps_to_average(guid_bpm_config_taps_to_average, 30);
cfg_int bpm_config_seconds_to_reset_average(guid_bpm_config_seconds_to_reset_average, 5);

HWND bpm_preferences_page::create(HWND parent)
{
	return Create(parent);
}

const char * bpm_preferences_page::get_name()
{
	return "BPM Auto Analysis";
}

GUID bpm_preferences_page::get_guid()
{
	return guid_bpm_preferences;
}

GUID bpm_preferences_page::get_parent_guid()
{
	return preferences_page::guid_tools;
}

void bpm_preferences_page::reset()
{
	// General
	bpm_config_bpm_precision = BPM_PRECISION_1DP;
	bpm_config_bpm_tag = "BPM";
	// Auto
	bpm_config_seconds_to_read = 10;
	bpm_config_num_bpms_to_calc = 10;
	bpm_config_offset_pct_min = 20;
	bpm_config_offset_pct_max = 80;
	bpm_config_fft_window_size = FFT_SIZE_2048;
	bpm_config_fft_window_slide = FFT_SLIDE_AUTO;
	bpm_config_fft_window_type = FFT_WINDOW_HAMMING;
	bpm_config_bpm_min = 75;
	bpm_config_bpm_max = 195;
	bpm_config_interpolate_flux = true;
	// Manual
	bpm_config_taps_to_average = 30;
	bpm_config_seconds_to_reset_average = 5;
}

bool bpm_preferences_page::reset_query()
{
	return true;
}

LRESULT bpm_preferences_page::OnInitDialog(CWindow wndFocus, LPARAM lInitParam)
{
	wchar_t w_bpm_tag[256];

	// General
	// This list MUST match the enum ordering of fft_window_size_enum
	CComboBox bpm_precision_box = GetDlgItem(ID_CONFIG_BPM_PRECISION);
	bpm_precision_box.AddString(_T("Nearest 5"));
	bpm_precision_box.AddString(_T("Nearest 1"));
	bpm_precision_box.AddString(_T("1 Decimal"));
	bpm_precision_box.AddString(_T("2 Decimals"));
	bpm_precision_box.SetCurSel(bpm_config_bpm_precision);
	pfc::stringcvt::convert_ansi_to_wide(w_bpm_tag, 256, bpm_config_bpm_tag.get_ptr(), bpm_config_bpm_tag.length());
	SetDlgItemText(ID_CONFIG_BPM_TAG, w_bpm_tag);

	// Auto
	SetDlgItemInt(ID_CONFIG_SECONDS_TO_READ, bpm_config_seconds_to_read, true);
	SetDlgItemInt(ID_CONFIG_NUM_BPMS_TO_CALC, bpm_config_num_bpms_to_calc, true);
	SetDlgItemInt(ID_CONFIG_OFFSET_PCT_MIN, bpm_config_offset_pct_min, true);
	SetDlgItemInt(ID_CONFIG_OFFSET_PCT_MAX, bpm_config_offset_pct_max, true);
	
	// This list MUST match the enum ordering of fft_window_size_enum
	CComboBox fft_window_size_box = GetDlgItem(ID_CONFIG_FFT_WINDOW_SIZE);
	fft_window_size_box.AddString(_T("64"));
	fft_window_size_box.AddString(_T("128"));
	fft_window_size_box.AddString(_T("256"));
	fft_window_size_box.AddString(_T("512"));
	fft_window_size_box.AddString(_T("1024"));
	fft_window_size_box.AddString(_T("2048"));
	fft_window_size_box.AddString(_T("4096"));
	fft_window_size_box.AddString(_T("8192"));
	fft_window_size_box.SetCurSel(bpm_config_fft_window_size);

	CComboBox fft_window_slide_box = GetDlgItem(ID_CONFIG_FFT_WINDOW_SLIDE);
	fft_window_slide_box.AddString(_T("64"));
	fft_window_slide_box.AddString(_T("128"));
	fft_window_slide_box.AddString(_T("256"));
	fft_window_slide_box.AddString(_T("512"));
	fft_window_slide_box.AddString(_T("1024"));
	fft_window_slide_box.AddString(_T("2048"));
	fft_window_slide_box.AddString(_T("4096"));
	fft_window_slide_box.AddString(_T("8192"));
	fft_window_slide_box.AddString(_T("Auto"));
	fft_window_slide_box.SetCurSel(bpm_config_fft_window_slide);

	CComboBox fft_window_type_box = GetDlgItem(ID_CONFIG_FFT_WINDOW_TYPE);
	fft_window_type_box.AddString(_T("Hamming"));
	fft_window_type_box.AddString(_T("Hanning"));
	fft_window_type_box.AddString(_T("Gausian"));
	fft_window_type_box.AddString(_T("Rectangular (None)"));
	fft_window_type_box.SetCurSel(bpm_config_fft_window_type);

	SetDlgItemInt(ID_CONFIG_BPM_MIN, bpm_config_bpm_min, true);
	SetDlgItemInt(ID_CONFIG_BPM_MAX, bpm_config_bpm_max, true);
	CheckDlgButton(ID_CONFIG_INTERPOLATE_FLUX, bpm_config_interpolate_flux ? BST_CHECKED : BST_UNCHECKED);

	// Manual
	SetDlgItemInt(ID_CONFIG_TAPS_TO_AVERAGE, bpm_config_taps_to_average, true);
	SetDlgItemInt(ID_CONFIG_SECONDS_TO_RESET_AVERAGE, bpm_config_seconds_to_reset_average, true);


	return 0;
}

void bpm_preferences_page::OnEditControlChange(UINT uNotifyCode, int nID, CWindow wndCtl)
{
	char a_bpm_tag[256];
	wchar_t w_bpm_tag[256];
	unsigned w_length = 0;
	unsigned a_length = 0;

	switch (nID)
	{
		case ID_CONFIG_BPM_TAG:
			w_length = GetDlgItemText(ID_CONFIG_BPM_TAG, w_bpm_tag, 256) + 1;
			a_length = pfc::stringcvt::estimate_wide_to_ansi(w_bpm_tag, w_length);
			bpm_config_bpm_tag.prealloc(a_length);
			pfc::stringcvt::convert_wide_to_ansi(a_bpm_tag, a_length, w_bpm_tag, w_length);
			bpm_config_bpm_tag = a_bpm_tag;
			break;
		case ID_CONFIG_SECONDS_TO_READ:
			bpm_config_seconds_to_read = GetDlgItemInt(ID_CONFIG_SECONDS_TO_READ, NULL, false);
			break;
		case ID_CONFIG_NUM_BPMS_TO_CALC:
			bpm_config_num_bpms_to_calc = GetDlgItemInt(ID_CONFIG_NUM_BPMS_TO_CALC, NULL, false);
			break;
		case ID_CONFIG_OFFSET_PCT_MIN:
			bpm_config_offset_pct_min = GetDlgItemInt(ID_CONFIG_OFFSET_PCT_MIN, NULL, false);
			break;
		case ID_CONFIG_OFFSET_PCT_MAX:
			bpm_config_offset_pct_max = GetDlgItemInt(ID_CONFIG_OFFSET_PCT_MAX, NULL, false);
			break;
		case ID_CONFIG_BPM_MIN:
			bpm_config_bpm_min = GetDlgItemInt(ID_CONFIG_BPM_MIN, NULL, false);
			break;
		case ID_CONFIG_BPM_MAX:
			bpm_config_bpm_max = GetDlgItemInt(ID_CONFIG_BPM_MAX, NULL, false);
			break;
		default:
			break;
	}
}

void bpm_preferences_page::OnComboBoxChange(UINT uNotifyCode, int nID, CWindow wndCtl)
{
	CComboBox box;

	switch (nID)
	{
		case ID_CONFIG_BPM_PRECISION:
			box = GetDlgItem(ID_CONFIG_BPM_PRECISION);
			bpm_config_bpm_precision = box.GetCurSel();
			break;
		case ID_CONFIG_FFT_WINDOW_SIZE:
			box = GetDlgItem(ID_CONFIG_FFT_WINDOW_SIZE);
			bpm_config_fft_window_size = box.GetCurSel();
			break;
		case ID_CONFIG_FFT_WINDOW_SLIDE:
			box = GetDlgItem(ID_CONFIG_FFT_WINDOW_SLIDE);
			bpm_config_fft_window_slide = box.GetCurSel();
			break;
		case ID_CONFIG_FFT_WINDOW_TYPE:
			box = GetDlgItem(ID_CONFIG_FFT_WINDOW_TYPE);
			bpm_config_fft_window_type = box.GetCurSel();
			break;
		default:
			break;
	}
}

LRESULT bpm_preferences_page::OnBnClicked(UINT uNotifyCode, int nID, CWindow wndCtl)
{
	switch (nID)
	{
		case ID_CONFIG_INTERPOLATE_FLUX:
			bpm_config_interpolate_flux = (IsDlgButtonChecked(ID_CONFIG_INTERPOLATE_FLUX) == BST_CHECKED);
			break;
		default:
			break;
	}

	return 0;
}

static preferences_page_factory_t<bpm_preferences_page> g_bpm_preferences_page;